// 
// Copyright(C) Rick Blommers 1999
// Written by Rick Blommers <rick@brainwave.demon.nl>
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MsQuery.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// The callback function
//
// NOTE: Only 1 instance of this object is allowed !!
//  
//
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
							// 'dit' is dutch for this :)
static CMSQuery *dit;		// points to the (last created) instance of this object

HDDEDATA CALLBACK CMSQuery::DdeCallback(UINT uType,UINT  uFmt, HCONV hconv, HSZ hsz1, 
    HSZ hsz2, HDDEDATA hdata, DWORD dwData1, DWORD dwData2) 
{ 
	// When it's zero, return (and assert in debug version)
	if (dit==NULL)
	{
		ASSERT(FALSE);	// there must be an object
		return NULL;
	}

	// Do the correct thing
    switch (uType) 
    { 
        case XTYP_REGISTER: 
        case XTYP_UNREGISTER: 
            return (HDDEDATA) NULL; 
 
        case XTYP_ADVDATA: 
            return (HDDEDATA) DDE_FACK; 
 
		// Is the transaction complete ?? And we can retrieve the SQL statement, we can get the query
        case XTYP_XACT_COMPLETE: 
			// If the transactionID is the same as the one created
			if (dwData1 == dit->m_TransActionID)
			{			
				// Reset the transaction id
				dit->m_TransActionID =0;
	
				// Get the query (if there's one !!) 
				dit->m_qryOK = TRUE;
				if (!dit->Request(dit->m_hConv, "ODBCSQLStatement", dit->m_sqlstatement)) 
				{	
					dit->m_qryOK = FALSE;
					dit->m_sqlstatement = "" ;
				}
				// When the query is ok, also get the connection string
				else 
				{
					dit->Request(dit->m_hConv, "ConnectionString", dit->m_connection);
				}

				if (dit->m_sqlstatement.GetLength()==0)
				{
					dit->m_qryOK = FALSE;
				}
			
				// We're done
				dit->m_dlg->PostMessage( WM_COMMAND, IDCANCEL, 0);

			}
            return (HDDEDATA) NULL; 
 
        case XTYP_DISCONNECT: 
            return (HDDEDATA) NULL; 
 
        default: 
            return (HDDEDATA) NULL; 
    } 
} 




//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//======================================================
// Construction
//======================================================
CMSQuery::CMSQuery()
{
	// Initialise everything at zero 
	m_idInst = 0;
	m_hszServName = 0;
	m_hszSysTopic = 0;
	m_hszQryTopic = 0;
	m_hProcess = 0;
	m_TransActionID = 0;
	dit = this;
	m_qryOK = FALSE;
}

//======================================================
// Destruction
//======================================================
CMSQuery::~CMSQuery()
{

	// Abandon transaction, when busy
	if (m_TransActionID) DdeAbandonTransaction(m_idInst, m_hConv, m_TransActionID);		

	// Also send the quit message 
	if (m_hConv) Execute(m_hConv, "[Exit(FALSE)]");	// Try to close the application normally
													// FALSE means closeing all channel related items

	// Kill the process
	if (m_hProcess)TerminateProcess(m_hProcess, 1);
  	if (m_idInst)	DdeUninitialize(m_idInst);			// This also kills all other instance related handles etc.

	// Change dit
	dit = NULL;										// dit is zero
}


//======================================================
// Create
//======================================================
BOOL CMSQuery::Create(CString & text, CString & connect, CDialog *pDlg)
{
	// Copy the pointer to the dialog:
	m_dlg = pDlg;

	// Return true when doen
	BOOL result = (DdeInitialize(&m_idInst, (PFNCALLBACK) DdeCallback, CBF_FAIL_EXECUTES|CBF_SKIP_ALLNOTIFICATIONS,0)==DMLERR_NO_ERROR);
	if (!result) return result;


	// Get the connection strings
	m_hszServName = DdeCreateStringHandle(m_idInst, "MSQuery", CP_WINANSI);   
	m_hszSysTopic = DdeCreateStringHandle(m_idInst, SZDDESYS_TOPIC, CP_WINANSI);

	// Connect to MSQuery 
	m_hConv = DdeConnect(m_idInst, m_hszServName, m_hszSysTopic, (PCONVCONTEXT) NULL);
	result = (m_hConv!=0);

	// couldn't connect, then MSQuery probably doesn't run...
	if (!result) 
	{
		// Get the location from the registry
		CString location;
		HKEY handle;
		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Shared Tools\\MSQuery", 0, KEY_QUERY_VALUE, &handle) ==  ERROR_SUCCESS)
		{
			unsigned long type=REG_SZ, size=_MAX_PATH;
			if (RegQueryValueEx(handle, "Path", 0, &type, (unsigned char*)location.GetBuffer(_MAX_PATH+1), &size)==ERROR_SUCCESS)
			{
				if (type == REG_SZ)
				{
					location.ReleaseBuffer();						
					result = TRUE;
				}
				else location.ReleaseBuffer(size);
			}	
			RegCloseKey(handle);
		}
		
		// Check if we've got the string
		if (!result) return result;

		// Try to start MSQuery
		STARTUPINFO si = 
		{
		    sizeof(STARTUPINFO),	//DWORD   cb; 
			NULL,					//LPTSTR  lpReserved; 
			NULL,					//LPTSTR  lpDesktop; 
			NULL,					//LPTSTR  lpTitle; 
			0, 0,					//DWORD   dwX; dwY
			0, 0,					//DWORD   dwXSize; dwYSize; 
			0, 0,					//DWORD   dwXCountChars; dwYCountChars; 
			0,						//DWORD   dwFillAttribute; 
			STARTF_USESHOWWINDOW,	//DWORD   dwFlags; 
			SW_SHOWNORMAL,			//WORD    wShowWindow; 
			0,						//WORD    cbReserved2; 
			NULL,					//LPBYTE  lpReserved2; 
			0, 0, 0					//HANDLE  hStdInput; hStdOutput; hStdError; 
		};
		
		// Start the process
		PROCESS_INFORMATION pi;
		if (!CreateProcess(location, NULL, NULL, NULL, FALSE, NORMAL_PRIORITY_CLASS, NULL, NULL/*default dir*/, &si, &pi))
		{
			return FALSE;
		}

		// Wait till the process is ready (maximum is 1 minute)
		result = ((BOOL)WaitForInputIdle(pi.hProcess, _startupDelay)==0);
	
		// Now again try to make connection
		if (result)	m_hConv = DdeConnect(m_idInst, m_hszServName, m_hszSysTopic, (PCONVCONTEXT) NULL);
		else m_hConv=0;

		// No dde connection, exit 
		if (m_hConv==0) 
		{
			m_qryOK = FALSE;
			TerminateProcess(pi.hProcess, 1);
			return FALSE;
		}

		// Copy the process handle
		m_hProcess = pi.hProcess;
	}

	// Try to create a new query
	if (text.GetLength()==0)
	{
		Execute(m_hConv, "[ViewPane(3)]");				// Show also the criterea pane
		if (!Execute(m_hConv, "[UserControl(\"&Terug naar DBS\",1,TRUE)]",TIMEOUT_ASYNC,&m_TransActionID)) return FALSE;
	}
	// Try to open a the existing query
	else
	{
		// Call "[BuildODBC(" + m_connection + ")]"
		CString query;
		query = "[BuildODBC(\"";
		query += connect;
		query += "\")]";
		if (!Execute(m_hConv, query)) return FALSE;

		// Call "[LogOnNow(1)]"
		if (!Execute(m_hConv, "[LogonNow(1)]")) return FALSE;

		// Open the query
		query.Empty();
		query  = "[Open(\"";
		query += text;
		query += "\")]";
		if (!Execute(m_hConv, query)) return FALSE; 
		Execute(m_hConv, "[ViewPane(3)]");		// Show also the criterea pane

		// Add the back to application part in the menu
		if (!Execute(m_hConv, "[UserControl(\"&Return to Application\",1,FALSE)]",TIMEOUT_ASYNC,&m_TransActionID)) return FALSE;
	}
	return result;
}




//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// Visual Basic kind commands (to simplify DDE)
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


//======================================================
// DDERequest
//======================================================
BOOL CMSQuery::Request(HCONV conv, const char *itemname, CString & result, int delay)
{
	// Create the string handle
	DWORD res;
	HSZ	str = DdeCreateStringHandle(m_idInst, itemname, CP_WINANSI);
	if (str==0) return FALSE;

	// Do the client transaction
	HDDEDATA dta = DdeClientTransaction((LPBYTE)NULL, 0, conv, str, CF_TEXT, XTYP_REQUEST, delay, &res);
	if (dta==0) 
	{
		DdeFreeStringHandle(m_idInst, str);
		return FALSE;
	}

	// Get the data
	int size = DdeGetData(dta, NULL, 0, 0);
	char *buf = result.GetBuffer(size+1);
	DdeGetData(dta, (unsigned char*)buf, size, 0);	
	buf[size] = 0;
	result.ReleaseBuffer();

	// Free the string handle
	DdeFreeStringHandle(m_idInst, str);


	// Free the datahandle
	DdeFreeDataHandle(dta);

	return TRUE;
}

//======================================================
// Execute
//======================================================
BOOL CMSQuery::Execute(HCONV conv, CString command, int delay, LPDWORD res)
{
	// Dot the MSQuery execution
	HDDEDATA dta = DdeClientTransaction((LPBYTE)command.GetBuffer(1), command.GetLength(), conv, 0, 0, XTYP_EXECUTE, delay, res);
	if (dta==0) return FALSE;

	// Free the data
	DdeFreeDataHandle(dta);
	return TRUE;
}


